<?php
include_once "elacset.php"; 

assert_right(RT_FUNCTIONS, RI_SMENA_GRAPHICS);

class DayField extends StringField
{ public $required=false;
  public $size=1;
  public $date;

  function __construct($d)
  { $this->date = $d;
	$this->caption=$d->format('j');
    $this->name='D'.$this->caption; 
	$this->td_class[]='DAY';

	$w=intval($d->format('w'));
	
	if(in_array($w, [0,6]))	 $this->th_class[]='HOL';    

	$names = ['', '', '', '', '', '', ''];
	$this->title=$d->format('d.m.Y').' '. $names[$w];	
  }
}

new class extends TDataset
{
 private $start, $end;
 private $days=[]; //   
 private $startend;
 
 function init()
 {
  $this->fields=
  [
    new class extends IntField
    { public $name  = 'SMENA'; 
 	  public $data = true;
	  public $visible=false;	  
	  function restrict($v, $action) { assert_right(RT_SMENA, $v);  }	  
    },
	new class extends StringField
	{ public $name  = 'NAME';  
      public $caption =  '';
	  public $readonly = true;
	} 
  ];
  
  $this->start=new DateTimeImmutable($this->params['MONTH']);    
  $this->end = $this->start->modify("first day of next month");
  
  $this->startend = ['start' => $this->start->format('d.m.Y'),  
                     'end'   => $this->end->format('d.m.Y')];
  
  
  for($d=$this->start; $d<$this->end; $d=$d->modify('+1 day'))
     $this->days[] = $this->fields[] = new DayField($d);	 
	 
  $this->select_sql  = "select smena,name from D_SMENA ".sql_where("where smena",RT_SMENA)." order by name";
  $this->refresh_sql = "select smena,name from D_SMENA where smena=:smena";  
  
 }
 

 function fetched(&$row)
 {
  foreach($this->prepare_execute('select letter,extract(day from data) as d from D_SMENA_GRAPHIC 
                                  where smena=:smena and data>=:start and data<:end', $row+$this->startend) as $let)								  
    $row['D'.$let['D']]=$let['LETTER'];    								  
 }
 
 function on_update($params)
 { 
  $old=[];  
  foreach($this->prepare_execute('select data, letter from D_SMENA_GRAPHIC where smena=:smena and data>=:start and data<:end', $params + $this->startend) 
           as [$data, $letter])     
    $old[(new DateTimeImmutable($data))->format('Y-m-d')] = $letter;
  
  foreach($this->days as $f)
  {  $p = ['letter' => $params[$f->name],
           'data'   => $f->date->format('Y-m-d')];
		  
     if(is_null($p['letter']) && isset($old[$p['data']]))
	   $this->prepare_execute('delete from D_SMENA_GRAPHIC where smena=:smena and data=:data', $params +$p);
		  
     if(!is_null($p['letter']) && !isset($old[$p['data']]))		  
       $this->prepare_execute('insert into D_SMENA_GRAPHIC (smena,data,letter) values (:smena,:data,:letter)', $params +$p);
		  
     if(!is_null($p['letter']) && isset($old[$p['data']]) && $p['letter']!=$old[$p['data']])		  
   	   $this->prepare_execute('update D_SMENA_GRAPHIC set letter=:letter where smena=:smena and data=:data', $params +$p);
  }
     
  $this->reload($params, 'update');   
 } 
 
};