<?php
include_once "elacset.php"; 
include_once "WF7.php"; 
include_once "colors.php"; 

assert_right(RT_FUNCTIONS, RI_LEAVES);


class LeaveDateField extends TField
{
   function draw_text($v, $row) 
   { if(!is_null($v))
	  { $dt = new DateTimeImmutable($v);
   	    echo $dt->format('d.m.Y');
		if($dt!=$dt->setTime(0,0))
           echo $dt->format(' G:i');
	  }  
   }
   
   function parse($v)
   {  if($v=='') return null;
            else return (new DateTime($v))->format('d.m.Y G:i:s');
   }
}


new class extends TDataset
{
 public $page_size=100;

 public $reverse=true;
 
 function init()
 {
  $this->fields=
  [
    new class extends GenIntField
    { public $name = 'ID'; 
      public $caption = '';
 	  public $data= true;
	  public $generator='D_GEN_LEAVES';
    },
    new class extends IntField 
	{ public $name = 'USER_REF'; 
	  public $data= true;
   	  public $visible = false;
	},		
    new class extends IntField 
	{ public $name = 'LEAVE_TYPE'; 
   	  public $visible = false;
	},	
    new class extends StringField
    { public $name = 'CODE';
      public $caption = '';
	  public $readonly = true;  	  
    },	
    new class extends StringField
    { public $name = 'FLAGS';
      public $caption = '';  	  
	  public $required = false;
    },	
    new class extends StringField
    { public $name = 'GROUPNAME';
      public $caption = '';  
	  public $readonly = true;
    },	
    new class extends StringField
    { public $name = 'TABEL_NUMBER';
      public $caption = '. ';  	  
	  public $readonly = true;
    },		
    new class extends StringField
    { public $name = 'USERNAME';
      public $caption = '';
	  public $readonly = true;  
    },		
    new class extends IntField
	{ public $name = 'A_LEVEL';
	  public $visible = false;
	  public $required = false;
	},	
	new class extends LeaveDateField
	{ public $name = 'START_TIME';
      public $caption = '';  
	},
	
    new class extends LeaveDateField
	{ public $name = 'END_TIME';
      public $caption = '';  
	},
    new class extends StringField
    { public $name = 'PRIM';
      public $caption = '';  	  
	  public $required = false;
    },	
	
    new class extends StringField
    { public $name = 'EDITOR_NAME';
      public $caption = '';  	  
	  public $readonly = true;
    },	

    new class extends DateField
    { public $name = 'CREATE_TIME';
      public $caption = '';  	  
	  public $format='d.m.y G:i';
	  public $readonly = true;
    }
  ];   


 $sel=
"select  first $this->page_size   
         l.id,l.user_ref,u.username,l.start_time,l.end_time, l.create_time,t.code, 
		 l.prim,u.tabel_number,l.leave_type,
		 t.flags,
         s.editor_name,
         g.groupname
from D_LEAVES l
left join USERS u on u.num=l.user_ref
left join D_LEAVE_TYPES t on l.leave_type=t.leave_type
left join D_EDITOR_SETTINGS s on s.editor=l.editor
left join GROUPS g on g.num=u.usergroup ";
  
  $this->select_sql  = $sel . '@filter order by id desc';
  $this->refresh_sql = $sel . 'where l.id=:id';
  
  $this->update_sql = 
  'update D_LEAVES set 
      start_time = :start_time,  end_time = :end_time,
      leave_type = :leave_type,
      prim = :prim,
      a_level=:a_level
   where  id = :id';
    
  $this->delete_sql = 'delete from D_LEAVES where id = :id';
  
  $this->insert_sql = 'insert into D_LEAVES (id, user_ref, start_time, end_time, editor, leave_type, prim, a_level)
                       values(:id,:user_ref, :start_time, :end_time, :editor, :leave_type,  :prim,  :a_level)';

    
 }

 public $filter=[];
 
 function build_filter(&$params)
 { 
   if(right_restricted(RT_GROUPS))   
     $this->filter[]=sql_where("u.usergroup",RT_GROUPS);  
   
   if(isset($params['USER']))
     $this->filter[]='l.user_ref=:user';
   
   if($this->filter) $params['@filter']='where '.implode(' and ',$this->filter);   
 }
 
 function before_select(&$params)
 {  $this->build_filter($params);     
 }
 
 function before_scroll(&$params)
 { $this->filter[]='l.id<:id';
   $this->build_filter($params);   
 }
 
 function before_insert(&$params)
 { global $editor;
   $params['EDITOR']=$editor;
 }
 
 
 
 function validate_also($params, $action)
 {
   if(is_null($params["A_LEVEL"])) return;
  
   $q=$this->prepare_execute("select id from D_LEAVES 
                              where  a_level is not null and  id!=:id and  user_ref=:user_ref and 
                              ((start_time between :start_time and :end_time) or
                               (start_time<=:start_time and end_time>=:start_time))", $params);

   if($id=$q->fetchColumn())  throw new Exception("       $id");
}
 
 
 }; //TDataset 