<?php
include_once "elacset.php"; 
include_once "dev_ip.php"; 

assert_right(RT_FUNCTIONS, RI_HARDWARE);

const TIMER_COUNT=8; 

new class extends TDataset
{
 protected $autoselect=true;
 protected $full_refresh=true; 

 function init()
 {
  $this->fields=
  [
    new class extends GenIntField
    { public $name = 'READER'; 
      public $data = true;
	  public $key = true;   
 	  public $visible= false;
	  public $generator='d_gen_devices';
    },

    new class extends IntField
    { public $name = 'COMPUTER'; 
      public $data = true;   
 	  public $visible= false;
    },
    new class extends StringField
    { public $name = 'ADR';
      public $caption = '';  
	  
     function draw_text($value, $row) 
	 { if(!is_null($value)) echo dev_addr_str($value); }
	 
	 function parse($v)
     { //if($v=='') return null;  
       if(!valid_dev_addr($v)) throw new Exception(' '); 	    
       return str_to_dev_addr($v);
     }	 
    },	
    new class extends IntField
    { public $name = 'SCHIT';
      public $caption = '';  
	  public $title = ' ';  
    },	
    new class extends StringField
    { public $name = 'READ_NAME';
      public $caption = '';  
  	  public $size=64;	  
    }	
  ];   
    
  $this->select_sql ='select * from D_DEVICES where computer=:computer order by iif(adr<0, -adr, adr),schit';   
  $this->refresh_sql='select * from D_DEVICES where reader=:reader';
  $this->delete_sql='delete from D_DEVICES where reader=:reader';
 }
 
function on_insert($params) 
{ 
  $this->prepare_input_params($params, 'insert'); 
  $this->do_update($params);
}

function on_update($params) 
{ $this->prepare_input_params($params, 'update'); 
  $this->do_update($params);
}

function do_update($params) 
{

if($params['READER_TYPE']>=3)
{
 for($i=0; $i<TIMER_COUNT; $i++)  $params["DEV_TIMER$i"]=(int)($params["DEV_TIMER$i"]*10);  

 if(in_array($params['RD_TYPE'], [9, 10]))
   $params['RD_PARAM'] =  ($params['RD_THRESHOLD']<<8) +  $params['RD_PROPABILITY'];
} 


foreach($this->update_sql as $i=>$sql)
{
 $q=$this->prepare_execute($sql, $params);
 if(!$q->rowCount())
    $this->prepare_execute($this->insert_sql[$i], $params); 
 if($params['READER_TYPE']<3) break;
}

$this->reload($params, 'update');

} //do_update

protected $update_sql=
[
'UPDATE D_DEVICES SET 
 ADR = :ADR, SCHIT = :SCHIT, READ_NAME = :READ_NAME, R1_01 = :R1_01, R1_2 = :R1_2,  R1_34 = :R1_34,  R1_5 = :R1_5,  R1_6 = :R1_6,
 R1_7 = :R1_7,  R1_OPENTIME = :R1_OPENTIME,  R1_MAXTIME = :R1_MAXTIME,  R2_5 = :R2_5,  NEED_SOUND = :NEED_SOUND,  READER_TYPE = :READER_TYPE,
 WARN_RELE = :WARN_RELE,  EXIT_WITH_OK = :EXIT_WITH_OK, TEST_REPEAT_ENTER = :TEST_REPEAT_ENTER, PERS_CODE_SERIA=:PERS_CODE_SERIA,
 WRITE_PHOTOID_EVENT=:WRITE_PHOTOID_EVENT
 WHERE READER = :READER',
 
'UPDATE D5_DEVICES SET 
 MODE = :DEV_MODE,  ENTER1 = :DEV_ENTER1,  ENTER2 = :DEV_ENTER2,  ENTER3 = :DEV_ENTER3,  ENTER4 = :DEV_ENTER4,  ENTER5 = :DEV_ENTER5,  ENTER6 = :DEV_ENTER6,  ENTER7 = :DEV_ENTER7,
 ENTER8 = :DEV_ENTER8,  ENTER9 = :DEV_ENTER9,  ENTER10 = :DEV_ENTER10,  ENTER11 = :DEV_ENTER11,  ENTER12 = :DEV_ENTER12,  TIMER0 = :DEV_TIMER0, TIMER1 = :DEV_TIMER1,
 TIMER2 = :DEV_TIMER2,  TIMER3 = :DEV_TIMER3,  TIMER4 = :DEV_TIMER4,  TIMER5 = :DEV_TIMER5, TIMER6 = :DEV_TIMER6, TIMER7 = :DEV_TIMER7,  UART_SPEED1 = :DEV_UART_SPEED1,
 UART_SPEED2 = :DEV_UART_SPEED2,  LPT_TYPE=:DEV_LPT_TYPE, PROGRAM=:DEV_PROGRAM
 WHERE  ADR = :ADR and COMPUTER = :COMPUTER', 
 
'UPDATE D5_SIDES SET 
 MODE = :SIDE_MODE, LOCK_RELE = :SIDE_LOCK_RELE, UNBLOCKED_DOOR_INFO = :SIDE_UNBLOCKED_DOOR_INFO,  TRIGGER_MODE = :SIDE_TRIGGER_MODE,  EM_LOCK = :SIDE_EM_LOCK
 WHERE  ADR = :ADR and COMPUTER = :COMPUTER and SIDE = :SIDE',
 
'UPDATE D5_READERS SET 
TYPE = :RD_TYPE, PHYS_ADDR = :RD_PHYS_ADDR, PARAM = :RD_PARAM, ID_MODE = :RD_ID_MODE, LOCAL_APB = :RD_LOCAL_APB, WARN_SOUND = :RD_WARN_SOUND, 
OPEN_SOUND = :RD_OPEN_SOUND, ALGO=:RD_ALGO
WHERE READER = :READER'];

protected $insert_sql=
[
'INSERT INTO D_DEVICES
(READER, COMPUTER, ADR, SCHIT, READ_NAME, R1_01, R1_2, R1_34, R1_5, R1_6, R1_7, R1_OPENTIME, R1_MAXTIME, R2_01, R2_2, R2_34, R2_5, R2_6,
 R2_OPENTIME, R2_MAXTIME, NEED_SOUND, READER_TYPE, WARN_RELE, EXIT_WITH_OK, TEST_REPEAT_ENTER, PERS_CODE_SERIA, WRITE_PHOTOID_EVENT)
VALUES
(:READER, :COMPUTER, :ADR, :SCHIT, :READ_NAME, :R1_01, :R1_2, :R1_34, :R1_5, :R1_6, :R1_7, :R1_OPENTIME, :R1_MAXTIME, 0, 0, 0, :R2_5, 0,
  0, 0, :NEED_SOUND, :READER_TYPE, :WARN_RELE, :EXIT_WITH_OK, :TEST_REPEAT_ENTER, :PERS_CODE_SERIA, :WRITE_PHOTOID_EVENT)',

'INSERT INTO D5_DEVICES
( ADR, COMPUTER, MODE,  ENTER1, ENTER2, ENTER3, ENTER4, ENTER5, ENTER6, ENTER7, ENTER8, ENTER9, ENTER10, ENTER11, ENTER12, TIMER0, TIMER1,
  TIMER2, TIMER3, TIMER4, TIMER5, TIMER6, TIMER7, UART_SPEED1, UART_SPEED2, LPT_TYPE, APB_WAIT, PROGRAM) VALUES 
(:ADR, :COMPUTER, :DEV_MODE, :DEV_ENTER1, :DEV_ENTER2, :DEV_ENTER3, :DEV_ENTER4, :DEV_ENTER5, :DEV_ENTER6, :DEV_ENTER7, :DEV_ENTER8, :DEV_ENTER9, :DEV_ENTER10, :DEV_ENTER11, :DEV_ENTER12, 
 :DEV_TIMER0, :DEV_TIMER1, :DEV_TIMER2, :DEV_TIMER3, :DEV_TIMER4, :DEV_TIMER5, :DEV_TIMER6, :DEV_TIMER7, :DEV_UART_SPEED1, :DEV_UART_SPEED2, :DEV_LPT_TYPE, 0, :DEV_PROGRAM)',
  
'INSERT INTO D5_SIDES
 (ADR, COMPUTER, SIDE, MODE, LOCK_RELE, UNBLOCKED_DOOR_INFO, TRIGGER_MODE, EM_LOCK)  VALUES
 (:ADR,:COMPUTER,:SIDE, :SIDE_MODE, :SIDE_LOCK_RELE,:SIDE_UNBLOCKED_DOOR_INFO, :SIDE_TRIGGER_MODE,:SIDE_EM_LOCK)',  
  
'INSERT INTO D5_READERS
 (READER, TYPE, PHYS_ADDR, PARAM, ID_MODE, LOCAL_APB, WARN_SOUND, OPEN_SOUND, ALGO) VALUES
 (:READER, :RD_TYPE, :RD_PHYS_ADDR, :RD_PARAM, :RD_ID_MODE, :RD_LOCAL_APB, :RD_WARN_SOUND, :RD_OPEN_SOUND, :RD_ALGO)'  
];
 
protected $error_map = 	[];  
 
 
 }; //TDataset 