<?php
include_once "elacset.php"; 
include_once "WF3.php"; 
include_once "colors.php"; 

assert_req_right();

class VisitDateField extends DateField
{
   public $format='d.m.y';

   function get_styles($value, $row)  
   {  if($row['ACTIVED'])  return ["background: var(--GlobalGreen)", "color: black"];  }
}


class ReqDataField extends StringField
{ 
  public $id;
  public $req_type;
  public $size=255;
  public $visible = false; 
 
  function get_insert_sql()
   { return "insert into D_REQ_DATA (req_id,type_id,data) values (:requisition,$this->id,:$this->name)"; }
}


new class extends TDataset
{
 public $page_size=100;

 public $reverse=true;
 
 private $guests_sql, $active_sql;

 private $filter=[];

 
 function init()
 {
  $this->fields=
  [
    new class extends GenIntField
    { public $name = 'REQUISITION'; 
      public $caption = '';
 	  public $data= true;
	  public $generator='D_GEN_REQUISITIONS';
    },
	new class extends IntField
	{ public $name = 'CARD_NUM'; 
      public $caption = '';
	  public $title = '  ';
	},
	new class extends BoolField
	{ public $name = 'HIDDEN'; 
    },	
    new class extends VisitDateField
    { public $name = 'VISIT_DATE';
      public $caption = '';  
	  public $title = '  ';
    },	
    new class extends VisitDateField
    { public $name = 'VISIT_LAST_DATE';
      public $caption = '.';  
      public $title = '  ';  
    },	
    new class extends IntField
    { public $name = 'RENCH_LEVEL';
    },		
    new class extends IntField
    { public $name = 'ACTIVED';
    },	
    new class extends IntField
    { public $name = 'GUEST';
    },	
    new class extends StringField
    { public $name = 'GUESTNAME';
      public $caption = ' ';  
    },	
    new class extends IntField
    { public $name = 'REQ_TYPE';
      public $required=false;
    },	
    new class extends StringField
    { public $name = 'RT_NAME';
      public $caption = ' ';  	  
	  
	  function get_styles($value, $row)  
	  { 
	    if(!is_null($row['RT_COLOR']))
	     return ["background: ".html_color($row['RT_COLOR']), "color: black"]; 	  		
	  }	
	  
	  function __construct()
	  {  global $db;
	     $this->visible=$db->query("select count(*) from D_REQ_TYPES")->fetchColumn();
	  }	    
    }, 
    new class extends StringField
    { public $name = 'USERORG';
      public $caption = '';  	  
    },	
    new class extends StringField
    { public $name = 'STATE_NAME';
      public $caption = '';  	  
	  
	  function get_styles($value, $row)  
	  {  global $req_state_colors;
	     return ["background: ".$req_state_colors[$row['STATE']], "color: black"]; 	  		
	  }	  	  
    },
    new class extends StringField
    { public $name = 'INITIATORGROUPNAME';
      public $caption = ' ';  	  
    },	
		
    new class extends StringField
    { public $name = 'INITIATORNAME';
      public $caption = '';  	  
    },	 
    new class extends IntField
    { public $name = 'INITIATOR';
    },		
    new class extends StringField
    { public $name = 'AIM';
      public $caption = ' ';  	  
    },	
	
    new class extends StringField
    { public $name = 'NOTES';
      public $caption = '';  	  
	  public $required=false;
    },	
	
    new class extends StringField
    { public $name = 'EDITOR_NAME';
      public $caption = '';  	  
    },	

    new class extends DateField
    { public $name = 'CREATED';
      public $caption = '';  	  
	  public $format='d.m.y G:i';
    }
  ];   


  $sel=  
"select    @first
           gd1.data_value as userorg, 
           (r.visit_date+r.visit_days-1) as visit_last_date,
		   iif(cast('now' as date) between r.visit_date and (r.visit_date+r.visit_days), 1, 0) as actived,
           r.*,u1.username as guestname,u2.username as initiatorname,g.groupname as initiatorgroupname,
           s.state_name,e.editor_name,
           rt.name as rt_name,
           rt.color as rt_color  
 from D_REQUISITIONS r
 left join D_REQ_TYPES rt on r.req_type=rt.id
 left join D_GUEST_DATA gd1 on gd1.guest=r.guest and gd1.data_type=1
 left join USERS u1 on r.guest=u1.num
 left join USERS u2 on r.initiator=u2.num
 left join GROUPS g on g.num=u2.usergroup
 left join D_TMP_CARDS c on c.requisition=r.requisition
 left join D_EDITOR_SETTINGS e on r.editor=e.editor
      join D_REQUISITION_STATES s on r.state=s.state ";

 $this->select_sql = $sel . '@filter  @sort';
 $this->refresh_sql = $sel .'where r.requisition=:requisition';
 
 $this->active_sql = $sel . "where r.visit_date<='today'  and  (r.visit_date + r.visit_days)>'today' and 
                             r.state in (0,1) and r.hidden=0 and c.card_num is null   
							 order by u1.username";	
 
 $this->guests_sql=
 "select r.requisition, c.card_num, u.username as guestname, gd1.data_value as userorg
  from D_TMP_CARDS c 
  join D_REQUISITIONS r on c.requisition=r.requisition
  join USERS u on r.guest=u.num
  left join D_GUEST_DATA gd1 on gd1.guest=r.guest and gd1.data_type=1
  order by  c.card_num";
 
 
	  
 $this->insert_sql = 	  
 [ "insert into D_REQUISITIONS ( requisition, req_type, guest, initiator, aim, visit_date, visit_days, notes, editor,rench_level)
                        values ( :requisition,:req_type, :guest, :initiator, :aim, :visit_date, :visit_days, :notes, :editor,:rench_level)",
   "insert into EVENTS (event_type,param1,param2,param3,param4) values (107,:editor,:guest,:initiator, :requisition)",
   "execute procedure D_UPDATE_GUEST_STATE(:guest)"
 ];
  
 $this->update_sql = 	    
 [
   "update D_REQUISITIONS 
    set req_type=:req_type, guest=:guest, initiator=:initiator, aim=:aim, visit_date=:visit_date, visit_days=:visit_days, 
	    notes=:notes, rench_level=:rench_level
    where requisition=:requisition",
   
    "delete from D_REQ_DATA where req_id=:requisition"
 ];
 

 $this->set_visible(['REQUISITION', 'VISIT_DATE', 'VISIT_LAST_DATE', 'GUESTNAME', 'RT_NAME', 'USERORG',
                     'STATE_NAME', 'INITIATORGROUPNAME', 'INITIATORNAME', 'AIM', 'NOTES', 'EDITOR_NAME', 'CREATED']); 

 $this->set_notreadonly(['REQUISITION', 'REQ_TYPE', 'GUEST', 'INITIATOR', 'AIM', 'VISIT_DATE', 'VISIT_LAST_DATE', 'NOTES', 'RENCH_LEVEL']);									  
  
	  
 foreach($this->db->query("select * from D_REQ_DATA_TYPES order by id") as $row)	  
 {  $fld= new ReqDataField;
    $fld->id = $row['ID'];
    $fld->name='DATA_'.$row['ID'];
	$fld->req_type=$row['REQ_TYPE'];
	$fld->required=$row['REQUIRED'];
	$fld->caption =$row['NAME'];
	$this->fields[]=$fld;	
 }
 
  global $editor;
  $this->params['EDITOR']=$editor; 
 } //init
 
 
 function before_select(&$params)
 { 
   $params['@first']="first $this->page_size"; 
   $params['@sort']='order by r.requisition desc';
 
   if(right_restricted(RT_GROUPS)) 
      $this->filter[]=sql_where("u2.usergroup",RT_GROUPS);  
   if($this->filter) $params['@filter']='where '.implode(' and ',$this->filter);
 }
 
 function before_scroll(&$params)
 { $this->filter[]='r.requisition<:requisition';
   $this->before_select($params);
 }
 
 function on_active($params) //   
 {
   $this->set_visible(['REQUISITION', 'GUESTNAME', 'USERORG']);
   $this->draw($this->prepare_execute($this->active_sql, $params));
 }
 
 function on_guests($params) // 
 {
   $this->fn['CARD_NUM']->data=true;
   $this->set_visible(['CARD_NUM', 'GUESTNAME', 'USERORG']);
   $this->draw($this->prepare_execute($this->guests_sql, $params));
 }
 
 
 function before_post(&$params, $action='*')
 { 
   assert_right(RT_FUNCTIONS, RI_NEWREQ);

   $row=$this->db->query("select guest_required_aim, cast('today' as date) as today from D_SETTINGS")->fetch();
   $this->fn['AIM']->required=$row['GUEST_REQUIRED_AIM'];
 
   $TODAY           = new DateTimeImmutable($row['TODAY']);
   $VISIT_DATE      = new DateTimeImmutable($params['VISIT_DATE']);
   $VISIT_LAST_DATE = new DateTimeImmutable($params['VISIT_LAST_DATE']);

   if($VISIT_LAST_DATE<$VISIT_DATE) throw new Exception("      ");
   if($VISIT_DATE<$TODAY)  throw new Exception("      ");
   
   $params['VISIT_DAYS']=date_diff($VISIT_DATE, $VISIT_LAST_DATE)->days+1;      
   
   foreach($this->fields as $f)
    if($f instanceof ReqDataField)
	 if(is_null($f->req_type) || $f->req_type==$params['REQ_TYPE'])
	 { $f->readonly=false;  
	   if(isset($params[$f->get_name($action)]))
        $this->update_sql[]=
		$this->insert_sql[]=$f->get_insert_sql();	   
	 }	   
	 else $f->readonly=true;  
 }
 
 function before_insert(&$params)
 {  $this->before_post($params, 'insert');
 }

 function validate_modify($params) //return req record
 { 
    $req=$this->prepare_execute('select * from D_REQUISITIONS where requisition = :requisition', $params)->fetch(PDO::FETCH_ASSOC);
    if($req['STATE']!=0)                  throw new Exception('    '); 
    if($req['EDITOR']!=$params['EDITOR']) throw new Exception('    '); 
    return $req; 
 }
 
 function before_update(&$params)
 { 
   $this->validate_modify($params);
   $this->before_post($params, 'update');
 } 
 
 function on_delete($params)
 {
  assert_right(RT_FUNCTIONS,RI_CANCELREQ);  
  $req=$this->validate_modify($params);
  $this->prepare_execute(
   [ 'update D_REQUISITIONS set state=2 where state=0 and requisition=:requisition',
     'insert into EVENTS (event_type,param1,param2,param3,param4) values (108,:editor,:guest,null,:requisition)',
     'execute procedure D_UPDATE_GUEST_STATE(:guest)'
   ], $req); 
  $this->on_refresh($req);
 }
 

 function on_givecard($params)
 {
   assert_right(RT_FUNCTIONS, RI_GIVECARD);   
   $this->set_notreadonly(['REQUISITION', 'CARD_NUM']);
   $this->prepare_input_params($params);  
  
  
   $params['CARD_CODE']=$this->prepare_execute("select card_code from D_TMP_CARDS where card_num=:card_num", $params)->fetchColumn();
   if(!$params['CARD_CODE'])   throw new Exception('  '); 

   if($name=$this->prepare_execute("select u.username from D_OWNERS o,USERS u where o.user_ref=u.num and o.card_code=:card_code", $params)->fetchColumn())
       throw new Exception("   : $name");   	   
   
   $req=$this->prepare_execute("select guest, rench_level, visit_date, 
                                       (visit_date+visit_days) as visit_end_date, state, cast('now' as date) as now
                                       from D_REQUISITIONS  
                                       where requisition=:requisition", $params)->fetch(PDO::FETCH_ASSOC);	   

   if($this->prepare_execute("select user_ref from D_OWNERS where user_ref=:guest", $req)->fetch())
	   throw new Exception("      ");

   if($req['STATE']>1)  throw new Exception("   ");	   
   if($req['NOW']<$req['VISIT_DATE'] ||
      $req['NOW']>$req['VISIT_END_DATE'] ) throw new Exception("     ");	   

   $params['COLOR']=$this->db->query("select req_defcolor from D_SETTINGS")->fetchColumn();   
   $params['O_NUMBER']=0;
 
   $this->prepare_execute(
   [ "insert into D_OWNERS (user_ref,acc_lev,act_date,das_date,o_number,card_code,tab_number,color)
                    values (:guest,:rench_level,:visit_date,:visit_end_date,:o_number,:card_code,:card_num,:color)",
     "update D_REQUISITIONS set state=1 where requisition=:requisition",
	 "update D_TMP_CARDS set requisition=:requisition where card_num=:card_num",
     "insert into EVENTS (event_type,param1,param2,param3,param4) values (109,:editor,:guest,null,:requisition)",
     "execute procedure D_UPDATE_GUEST_STATE(:guest)"
   ], $req+$params);	 
 }


 
 function on_extract($params)
 {
  assert_right(RT_FUNCTIONS, RI_GIVECARD);   
  $this->set_notreadonly(['CARD_NUM', 'HIDDEN']);
  $this->prepare_input_params($params);
 
  $req=$this->prepare_execute("select r.requisition,r.guest from D_TMP_CARDS c,D_REQUISITIONS r 
                               where c.requisition=r.requisition and c.card_num=:card_num", $params)->fetch(PDO::FETCH_ASSOC);	   
  if(!$req) throw new Exception("   ");	   

  $this->prepare_execute(
  [ "delete from D_OWNERS where user_ref=:guest",
    "insert into EVENTS (event_type,param1,param2,param3,param4) values (110,:editor,:guest,null,:requisition)",
    "update D_TMP_CARDS set requisition=null where card_num=:card_num",
    "execute procedure D_UPDATE_GUEST_STATE(:guest)",
	"update D_REQUISITIONS set hidden=:hidden where requisition=:requisition"
  ], $req+$params);	  
 }
 

 
 
 }; //TDataset 